package
{
    import loom.Application;
    import loom.gameframework.TimeManager;

    import loom.sound.Listener;
    import loom.sound.Sound;
    import loom2d.display.StageScaleMode;
    import loom2d.display.Image;
    import loom2d.textures.Texture;
    import loom2d.textures.TextureSmoothing;
    import loom2d.ui.TextureAtlasManager;
    import loom2d.display.Sprite;
    import loom2d.events.Event;
    import system.platform.Platform;
    import system.platform.PlatformType;

    import feathers.controls.Button;
    
    import loom.sound.Listener;


    import com.rockethands.View;
    import com.rockethands.Theme;
    import views.GameView;
    import views.TitleView;
    import constants.Constants;

    public class DungeonsOfDestiny extends Application
    {

        // Gets injected automatically before run() is called
        [Inject] private var _time_manager: TimeManager;
        private var _display: Sprite = new Sprite();
        private var _currentView: View;
        private var _contentWidth = Constants.GAMEWIDTH;//450;
        private var _contentHeight = Constants.GAMEHEIGHT;//580;
        private var _pixelScale = Constants.PIXELSCALE;
        
        private var _back_button: Button;


        //Views
        private var _game_view = new GameView();
        private var _title_view = new TitleView();


        override public function run():void
        {
            // Comment out this line to turn off automatic scaling.
            stage.scaleMode = StageScaleMode.LETTERBOX;
            //stage.scaleMode = StageScaleMode.NONE;

            // Don't interpolate pixels - rough pixel art look
            TextureSmoothing.defaultSmoothing = TextureSmoothing.NONE;
            
            SplashLoader.init("assets/images/RocketJams.png", stage, _time_manager, load);
        }

        private function load()
        {
            Console.print("Begin load");

            // Load the sprite atlas containing textures
            TextureAtlasManager.register("tiles", "assets/spritesheets/dod_tiles.xml");
            
            // Create a new theme - this initialises all bitmapfonts etc.
            new Theme(null, "font_16/font.fnt");

            // View initialisation
            //var views: Vector.<View> = new <View>[_game_view];
            //for each (var view: View in views)
            //{
            //    view.init();
            //}
            _game_view.init("assets/layout/game.lml");
            _title_view.init("assets/layout/title.lml");
            _game_view.parse_level_file();
            //_game_view.create_next_room(stage);

            // Transitions between views
            _title_view.on_start += function(){switch_view(_game_view);};
            _title_view.on_credits += function(){Console.print("Show credits here");};
            _title_view.on_back += function(){Console.print("Confirm quit here");};
            
            _game_view.on_back += function(){switch_view(_title_view);};

            _game_view.soundtrack = Sound.load("assets/sounds/contemplation 2.ogg");
            _game_view.soundtrack.setLooping(true);
            _game_view.sfx["step"] = Sound.load("assets/sounds/step.ogg");
            _game_view.sfx["step"].play();
            _game_view.sfx["win"] = Sound.load("assets/sounds/win.ogg");
            _game_view.sfx["playerdie_hole"] = Sound.load("assets/sounds/playerdeathfall.ogg");
            _game_view.sfx["playerdie_damage"] = Sound.load("assets/sounds/playerdeathdamage.ogg");
            _game_view.sfx["enemydie_hole"] = Sound.load("assets/sounds/enemydeathfall.ogg");
            _game_view.sfx["enemydie_damage"] = Sound.load("assets/sounds/enemydeathdamage.ogg");

            // Handle app pausing
            applicationActivated += on_activated;
            applicationDeactivated += on_deactivated;
            
            stage.addChild(_display);
            stage.addEventListener(Event.RESIZE, resize);

            // Back button for non-Android platforms
            _back_button = new Button();
            _back_button.width = 20;
            _back_button.height = 20;
            _back_button.defaultIcon = new Image(Texture.fromAsset("assets/icons/iconBack.png"));
            _back_button.addEventListener(Event.TRIGGERED, on_back);
            if (Platform.getPlatform() != PlatformType.ANDROID) 
                stage.addChild(_back_button);


            Console.print("End load");

            switch_view(_title_view);
            //switch_view(_game_view);

        }

        private function on_back(e:Event):void 
        {
            _currentView.on_back();
        }
        
        /**
         * Mute sounds when the app is paused
         */
        private function on_deactivated():void 
        {
            Listener.setGain(0);
            _game_view.deactivate();
        }
        
        /**
         * Unmute sounds when the app is resumed
         */
        private function on_activated():void 
        {
            Listener.setGain(1);
            _game_view.activate();
        }

        /**
         * Responsive layout! Scale display so it's contained within stage
         */
        private function resize(e:Event = null)
        {
            var scale:Number;
            if (stage.stageWidth/stage.stageHeight < _contentWidth/_contentHeight) {
                scale = stage.stageWidth/_contentWidth;
            } else {
                scale = stage.stageHeight/_contentHeight;
            }
            // Scale to whole multiples, unless it's too small
            if (stage.stageWidth >= _contentWidth) {
                _display.scale = Math.max(1, Math.floor(_pixelScale*scale));
            } else {
                _display.scale = _pixelScale * scale;
            }
            _back_button.scale = _display.scale;
            var w = stage.stageWidth/_display.scale;
            var h = stage.stageHeight/_display.scale;
            _currentView.resize(w, h);
        }

        /**
         * State machine transition for views
         */
        private function switch_view(newView:View)
        {
            if (_currentView) _currentView.exit();
            _currentView = newView;
            _currentView.enter(_display);
            switch (_currentView) {
                //case intro:
                //case game:
                //    _back_button.visible = false;
                //    break;
                default: _back_button.visible = true; break;
            }
            resize();
        }
        
        /**
         * Pass on onTick to the current view
         */
        override public function onTick()
        {
            if (_currentView) _currentView.tick();
            return super.onTick();
        }
        
        /**
         * Pass on onFrame to the current view
         */
        override public function onFrame()
        {
            if (_currentView) _currentView.render();
            return super.onFrame();
        }



    }
}
