package com.rockethands
{   
    public delegate ViewCallback(): void;
    
    import loom2d.display.DisplayObjectContainer;
    import constants.*;
    import types.*;
    import loom.lml.LML; 
    import loom.lml.LMLDocument; 
    import loom.css.StyleSheet;
    import loom2d.events.Event;
    import loom2d.events.ResizeEvent;
    import loom2d.events.KeyboardEvent;
    import loom2d.textures.Texture;
    import loom2d.textures.TextureSmoothing;
    import loom2d.display.Stage;


    /**
    * base view class. convenience callbacks to trigger transitions
    * and sequence adding / removing from parent
    */
    
    class View extends DisplayObjectContainer
    {
        public static var activeViews: Vector.<View> = new Vector.<View>();
        public static var device_orientation: Orientation_t = Orientation_t.VERTICAL;
        // This only exists because for some reason Views don't have access
        // to the stage in the init method (and you might want to do
        // something there), but they do later on in the on_enter method
        // etc. So we make sure we set this at the beginning of the 
        // application, and in init methods, always use init_stage instead
        // of stage when getting widths/heights etc. A bit of a kludge, 
        // but hacks for now.
        public static var init_stage: Stage = null;

        public var on_enter: ViewCallback;
        public var on_exit: ViewCallback;
        public var on_back:ViewCallback;

        public var h_layout: StyleSheet;
        public var v_layout: StyleSheet;
        public var lml_source: String;
        public var inited: Boolean;

        public function View()
        {
            inited = false;
            h_layout = null;
            v_layout = null;
        }

        public function init(lml: String = null, hstyle: String = null, vstyle: String = null)
        {
            inited = true;
            if (lml)
            {
                lml_source = lml;
                Console.print("binding lml: " + lml);
                var doc = LML.bind(lml, this);
                Console.print("finished binding lml");
            }
            
            if (hstyle != null)
            {
                h_layout = new StyleSheet(hstyle);
                h_layout.source = hstyle;
                h_layout.onUpdate += style_updated;
            }
            if (vstyle != null)
            {
                v_layout = new StyleSheet(vstyle);
                v_layout.source = vstyle;
                v_layout.onUpdate += style_updated;
            }
        }
   
        public function tick() {}
        public function render() {}
        public function resize(width: Number, height: Number){}

        public static function relayout_all()
        {
            for each (var v: View in View.activeViews)
            {
                //v.relayout(o);
                v.relayout();
            } 
        }

        public function style_updated(stylesheet: StyleSheet) : void
        {
            //Console.print("Sylesheet updated: " + stylesheet.name);
            relayout();
        }

        public function update(dt: Number) : void
        {
        }
    
        public function enter(owner: DisplayObjectContainer): void
        {
            Console.print("Started entering scene: " + this.getTypeName());
            owner.removeChild(this); //just in case we are already in the parent.
            owner.addChild(this);
            //stage.addEventListener(ResizeEvent.RESIZE, onOrientationChanged);
            stage.addEventListener(KeyboardEvent.BACK_PRESSED, backPressed);
            on_enter();

            activeViews.push(this);
            Console.print("Finished entering scene");
        }

        private function backPressed(e:KeyboardEvent):void 
        {
            on_back();
        }


        public function exit(): void
        {
            //stage.removeEventListener(ResizeEvent.RESIZE, onOrientationChanged);
            if (stage) stage.removeEventListener(KeyboardEvent.BACK_PRESSED, backPressed);

            if (parent)
            {
                parent.removeChild(this);
                on_exit();
            }
            activeViews.remove(this);
        }

        public function relayout() : void
        {
            //var o: String = Cocos2D.getOrientation();
            //var o: String = GLYF_CONSTANTS.ORIENTATION_PORTRAIT;
            //if (o == GLYF_CONSTANTS.ORIENTATION_LANDSCAPE)
            if (device_orientation == Orientation_t.HORIZONTAL)
            {
                if (h_layout != null)
                {   
                    Console.print("Relaying out [" + lml_source + "] for horizontal: " + h_layout.name);
                    //var hdoc = LML.bind(h_layout, this);
                    //var hdoc = LML.bind(h_layout, this);
                    this.applyStyle(h_layout);
                }
            }
            else
            {
                if (v_layout != null)
                {
                    //var vdoc = LML.bind(v_layout, this);
                    //var vdoc = LML.bind(v_layout, this);
                    Console.print("Relaying out [" + lml_source + "] for vertical: " + v_layout.name);
                    this.applyStyle(v_layout);
                }
            }

        }

        //public function edgebutt(subject: CCSprite, to: CCSprite, edge: Edge_t)
        //{
        //}
        protected function onOrientationChanged(e: ResizeEvent)
        {
            Console.print("orientation changed: width: " + e.width + " height: " + e.height + ".");
            Console.print("Native width: " + stage.nativeStageWidth + " height: " + stage.nativeStageHeight);
            Console.print("Stage width: " + stage.stageWidth + " height: " + stage.stageHeight);
            
            var o: Orientation_t;
            
            //if (Cocos2D.getOrientation() == "ORIENTATION_PORTRAIT")
            //if (e.width < e.height)
            if (stage.nativeStageWidth < stage.nativeStageHeight)
                o = Orientation_t.VERTICAL;
            else
                o = Orientation_t.HORIZONTAL;

            if (device_orientation != o)
            {
                device_orientation = o;
                relayout_all();
            }
        }
    }

}
