package dungeonsofdestiny.game
{
    import types.TileEffect_t;
    
    import loom2d.display.DisplayObjectContainer;
    import loom2d.display.Sprite;
    import loom2d.display.Stage;
    import loom.sound.Listener;
    import loom.sound.Sound;

    import constants.Constants;
    
    
    import loom2d.math.Point;


    import dungeonsofdestiny.game.RoomLayer;


    import dungeonsofdestiny.game.tiles.*;
    import dungeonsofdestiny.game.enemies.*;

    class Room extends DisplayObjectContainer
    {
        //Rooms can be any grid dimensions
        public var dim_x: int;
        public var dim_y: int;
        private var _stage: Stage;
        private var _tile_display = new Sprite();
        public var tile_width: int;
        public var tile_height: int;
        private var _stage_width: int = Constants.GAMEWIDTH;
        private var _stage_height: int = Constants.GAMEHEIGHT;

        public var player: Player;
        public var enemies: Vector.<Enemy> = new Vector.<Enemy>;

        private var _bottom_layer: RoomLayer;
        private var _middle_layer: RoomLayer;
        private var _top_layer: RoomLayer;
        public var room_layers: Vector.<RoomLayer>;

        public var player_move_direction: Point;
        public var shared_sfx: Dictionary.<String, Sound>;

        public function Room(stage_: Stage, definition: Vector.<String>, sfx: Dictionary.<String, Sound>)
        {
            super();
            shared_sfx = sfx;
            _stage = stage_;
            Console.print("In room, _stage is : ", _stage);
            addChild(_tile_display);
                        
            //Dimensions from the definition
            dim_x = definition[0].length;
            dim_y = definition.length;


            _bottom_layer = new RoomLayer(this, dim_x, dim_y);
            _middle_layer = new RoomLayer(this, dim_x, dim_y);
            _top_layer = new RoomLayer(this, dim_x, dim_y);
            room_layers = new <RoomLayer>[_bottom_layer, _middle_layer, _top_layer];

            addChild(_bottom_layer);
            addChild(_middle_layer);
            addChild(_top_layer);
            
            tile_width = _stage_width/dim_x;
            tile_height = _stage_height/dim_y;
            create_room_tiles(definition);
        }

        public function teardown_room()
        {
            for (var j = 0; j < dim_y; j++)
            {
                for (var i = 0; i < dim_x; i++)
                {
                    for each (var layer: RoomLayer in room_layers)
                    {
                        var tile = layer.get_tile_at(i, j);
                        if (tile)
                        {
                            layer.remove_tile(tile);
                        }
                    }
                }
            }
            //removeChild(_bottom_layer);
            //removeChild(_middle_layer);
            //removeChild(_top_layer);

        }

        public function play_sfx(sname: String)
        {
            var s: Sound = shared_sfx[sname];
            if (s)
                s.play();
            else
                Console.print("Sound " + sname + " not found!");
        }

        public function create_room_tiles(definition: Vector.<String>)
        {
        
            Console.print("Room dimensions " + dim_x + "x" + dim_y + ". Tile width: " + tile_width + ", height" + tile_height);
            
            for (var j = 0; j < dim_y; j++)
            {
                for (var i = 0; i < dim_x; i++)
                {
                    //Console.print("j is " + j + ", i is " + i);
                    var line: String = definition[j];

                    //_bottom_layer.add_tile(new BorderNW(null, this, 0, 0);
                    //_middle_layer.add_tile(new Player(null, this), 0, 0);

                    switch (line.charAt(i))
                    {   
                        case '#':
                            create_border_tile(i, j);
                            break;
                        case ' ':
                            create_ground(i, j);
                            //_bottom_layer.add_tile(new Ground(null, this), i, j);
                            break;
                        case '>' :
                            _bottom_layer.add_tile(new DoorDest(null, this), i, j);
                            break;
                        case '<' :
                            _bottom_layer.add_tile(new DoorSrc(null, this), i, j);
                            break;
                        case '@':
                            _bottom_layer.add_tile(new Ground(null, this), i, j);
                            player = new Player(null, this);
                            _middle_layer.add_tile(player, i, j);
                            break;
                        case 's':
                            create_enemy(new Scorpion(null, this), i, j);
                            break;
                        case 'w':
                            create_enemy(new Wasp(null, this), i, j);
                            break;

                        case 'S':
                            create_enemy(new Spider(null, this), i, j);
                            break;
                        case 'D':
                            create_enemy(new Demon(null, this), i, j);
                            break;

                        case 'O':
                            _bottom_layer.add_tile(new Ground(null, this), i, j);
                            _middle_layer.add_tile(new Hole(null, this), i, j);
                            break;
                            
                        default:
                            Console.print("Room does not know how to parse \"" +  definition[j].charAt(i));
                            break;
                    }
                }
            }
        }

        public function create_border_tile(i: int, j: int)
        {
            if (j==0)
            {
                if (i == 0)
                    _bottom_layer.add_tile(new BorderNW(null, this), i, j);
                else if (i == dim_x-1)
                    _bottom_layer.add_tile(new BorderNE(null, this), i, j);
                else
                    _bottom_layer.add_tile(new BorderN(null, this), i, j);
            }
            else if (j==dim_y-1)
            {
                if (i == 0)
                    _bottom_layer.add_tile(new BorderSW(null, this), i, j);
                else if (i == dim_x-1)
                    _bottom_layer.add_tile(new BorderSE(null, this), i, j);
                else
                    _bottom_layer.add_tile(new BorderN(null, this), i, j);
            }
            else
            {
                if (i == 0)
                    _bottom_layer.add_tile(new BorderW(null, this), i, j);
                else if (i == dim_x-1)
                    _bottom_layer.add_tile(new BorderW(null, this), i, j);
                else
                    _bottom_layer.add_tile(new BorderC(null, this), i, j);
                    
            }

        }

        public function create_enemy(enemy: DungeonEntity, i: int, j: int)
        {
             _middle_layer.add_tile(enemy, i, j);
             create_ground(i, j);
             enemies.push(enemy);
        }

        public function create_ground(i: int, j: int)
        {
            _bottom_layer.add_tile(new Ground(null, this), i, j);
        }


        public function check_tile_in_position(from_dirx: int, fromdiry: int, posx: int, posy: int): int
        {
            var effect: int = 0;

            for each (var layer: RoomLayer in room_layers)
            {
                var tile: RoomTile = layer.get_tile_at(posx, posy);
                if (tile)
                    effect += tile.effect;
            }
            return effect;
            
        }

        /*
        public function on_touch(te: TouchEvent): void
        {
            // Filter to only consider new touches.
            var point:Point;
            
            var touch = te.getTouch(_stage, TouchPhase.BEGAN);
            if (touch)
            {
                point = touch.getLocation(_stage);
                //trace("Touch (,",touch.id,") x:", point.x, " y:", point.y);
                _movement_sensor.addSample(point);
            }

            touch = te.getTouch(_stage, TouchPhase.MOVED);
            if (touch)
            {
                point = touch.getLocation(_stage);
                //trace("MOVED (",touch.id,") x:", point.x, " y:", point.y);
                _movement_sensor.addSample(point);
            }

            touch = te.getTouch(_stage, TouchPhase.ENDED);
            if (touch)
            {
                point = touch.getLocation(_stage);
                //trace("ENDED (",touch.id,") x:", point.x, " y:", point.y);
                _movement_sensor.reset();
            }
        }*/

        /*
        public function on_key(ke: KeyboardEvent) : Point
        {
            //Console.print(ke.keyCode);
            var dir = new Point();
            switch (ke.keyCode)
            {
                case 82: //up
                    dir.x = 0;
                    dir.y = -1;
                    //process_sense_direction(dir);
                    break;
                case 79: //right
                    dir.x = 1;
                    dir.y = 0;
                    //process_sense_direction(dir);
                    break;
                case 81: //down
                    dir.x = 0;
                    dir.y = 1;
                    //process_sense_direction(dir);
                    break;
                case 80: //left
                    dir.x = -1;
                    dir.y = 0;
                    //process_sense_direction(dir);
                    break;
            }
            return dir;
        }
        */

        //public function get_player_move() : Point
        //{
        //    return _player_move_direction;
        //}

        

        public function get_x_position(grid_x: int) : int
        {   
            return tile_width * grid_x;
        }

        public function get_y_position(grid_y: int) : int
        {
            return tile_height * grid_y;
        }

    }
}
