package dungeonsofdestiny.game
{
    import dungeonsofdestiny.game.Room;
    import dungeonsofdestiny.game.RoomTile;
    import dungeonsofdestiny.game.RoomLayer;

    import loom2d.display.DisplayObjectContainer;


    class RoomLayer extends DisplayObjectContainer
    {
        private var _room: Room;

        private var _grid: Vector.<Vector.<RoomTile> > = new Vector.<Vector.<RoomTile> >;
        
        public function RoomLayer(room: Room, dim_x: int, dim_y: int)
        {
            super();
            _room = room;
            init_grid(dim_x, dim_y);
        }

        public function init_grid(dim_x: int, dim_y: int)
        {
            for (var i: int = 0; i < dim_x; i++)
            {
                _grid.push(new <RoomTile>); //Push a new vector
                for (var j: int = 0; j < dim_y; j++)
                {
                    _grid[i].push(null);
                }
            }
        }

        public function add_tile(tile: RoomTile, tx: int, ty: int)
        {
            //Console.print("adding tile to: " + tx + ", " + ty);
            _grid[tx][ty] = tile;
            tile.set_room_layer(this);
            addChild(tile.display_image);
            tile.reset_position(tx, ty);
        }

        public function remove_tile(tile: RoomTile)
        {
            _grid[tile.tx][tile.ty] = null;
            removeChild(tile.display_image);
            
        }

        public function move_tile(tile: RoomTile, tx: int, ty: int)
        {
            Console.print("Moving from " + tile.tx + ", " + tile.ty + " to " + tx + ", " + ty);
            _grid[tile.tx][tile.ty] = null;
            _grid[tx][ty] = tile;
            tile.reset_position(tx, ty);
        }

        public function get_tile_at(posx: int, posy: int): RoomTile
        {
            if (posx < 0 || posx >= _grid.length)
                return null;
            if (posy < 0 || posy >= _grid[posx].length)
                return null;
            return _grid[posx][posy];
        }
    }
}
