package views
{
    import loom.sound.Listener;
    import loom.sound.Sound;
    import loom2d.events.Event;
    import loom2d.events.TouchEvent;
    import loom2d.events.Touch;
    import loom2d.events.KeyboardEvent;
    import loom2d.events.TouchPhase;

    import loom2d.animation.Juggler;
    import loom2d.textures.Texture;
    import loom2d.display.DisplayObject;
    import loom2d.display.DisplayObjectContainer;
    import loom.platform.Mobile;
    import loom.platform.UserDefault;
    import loom2d.display.Stage;
    import constants.Constants;

    import feathers.controls.Button;
    import feathers.controls.Label;
    import feathers.display.OffsetTiledImage;

    import com.rockethands.MovementSensor;

    import dungeonsofdestiny.game.Room;
    import dungeonsofdestiny.game.Player;
    import dungeonsofdestiny.game.enemies.*;
    import dungeonsofdestiny.game.tiles.*;

    import dungeonsofdestiny.game.DungeonEntity;
    import dungeonsofdestiny.game.DungeonEntityMove;
    import loom2d.math.Point;
    import types.TileEffect_t;

    


    import com.rockethands.View;

    class GameView extends View
    {
        
        private var _paused:Boolean = false;
        private static const MUTE_NONE  = 0;
        private static const MUTE_MUSIC = 1;
        private static const MUTE_ALL   = 2;
        private var _muteMode = MUTE_NONE;

        private static const STATE_SLEEP   = 0;
        private static const STATE_GAME    = 1;
        private static const STATE_QUIT    = 2;
        private static const STATE_ENDING  = 3;
        private static const STATE_DEMO    = 4;
        private var _state = STATE_SLEEP;


        // User interface from LML
        [Bind] public var esc: Button;
        [Bind] public var mute: Button;
        [Bind] public var timeDisplay: Label;
        [Bind] public var lastDisplay: Label;
        [Bind] public var multiDisplay: Label;
        [Bind] public var scoreDisplay: Label;

        private var background:OffsetTiledImage;

        /** Delta time, how long each game tick lasts */
        private var _dt:Number = 1/60;
        private var _t:Number;
        
        /** Custom juggler used for most animations, enables easy pausing */
        private var _juggler:Juggler = new Juggler();

        private var _roomdefs : Vector.<Vector.<String> > = new Vector.<Vector.<String> >;
        private var _current_room_number = 1; //Needs to start here
        public var current_room: Room;

        // Sounds
        public var soundtrack: Sound;
        public var sfx: Dictionary.<String, Sound> = new Dictionary.<String, Sound>();
        private var _movement_sensor: MovementSensor;


        public function init(lml: String = null, hstyle: String = null, vstyle: String = null)
        {
            super.init(lml, hstyle, vstyle);
            mute.addEventListener(Event.TRIGGERED, switchMuteMode);
            // Get saved mute mode (or default if none exists)
            _muteMode = UserDefault.sharedUserDefault().getIntegerForKey("muteMode", _muteMode);
            _movement_sensor = new MovementSensor();
            //_movement_sensor.on_sense_direction += process_sense_direction;
        }

        public function parse_level_file()
        {
            var filename = "assets/" + Constants.LEVELS_FILE;
            var level_file_contents = File.loadTextFile(filename);
            Console.print("Level text: \n" + level_file_contents);
            var lines: Vector.<String> = level_file_contents.split("\n");
            Console.print(lines.length);
            //The space between >> is needed here to avoid compiler confusion
                        //Split into rooms
            var roomdef: Vector.<String> = new <String>;
            for each (var line: String in lines)
            {
                //If we encounter the blank line between
                //levels, push the current string and start another
                if (line.length == 1)
                {
                    _roomdefs.push(roomdef);
                    Console.print("Finished parsing a room");
                    roomdef = new <String>;
                }
                else
                {
                    roomdef.push(line);
                }
            }

        }

        public function submit_move(tile: DungeonEntity, xsteps: int, ysteps: int) : Boolean
        {
            var would_move: Boolean = false;
            //check if we can move without being stopped.
            //check if we can move without dying
            Console.print("Move request from " + tile.tx + ", " + tile.ty + " to " + tile.tx + xsteps + ", " + tile.ty + ysteps);
            var cumulative_effects: int =  current_room.check_tile_in_position(0, 0, tile.tx + xsteps, tile.ty + ysteps);
            Console.print("Effect at " + (tile.tx + xsteps) + ", " + (tile.ty + ysteps) + " = " + cumulative_effects);
            
            if (TileEffect_t.has_effect(cumulative_effects, TileEffect_t.WIN))
            {
                if (tile is dungeonsofdestiny.game.Player)
                {
                    current_room.play_sfx("win");
                    Console.print("processing player win");
                    //You won the level
                    wipe_current_room();
                    create_next_room(stage);
                }
            }

            if (TileEffect_t.has_effect(cumulative_effects, TileEffect_t.DAMAGE))
            {
                tile.die();
                if (tile is dungeonsofdestiny.game.Player)
                {
                    current_room.play_sfx("playerdie_damage");
                    //The level needs to end! Game over!
                    wipe_current_room();
                }
                else
                {
                    current_room.play_sfx("enemydie_damage");
                }

            }
            else if (TileEffect_t.has_effect(cumulative_effects, TileEffect_t.FALL))
            {
                tile.die();
                if (tile is dungeonsofdestiny.game.Player)
                {
                    current_room.play_sfx("playerdie_hole");
                    //The level needs to end! Game over!
                    wipe_current_room();
                }
                else
                {
                    current_room.play_sfx("enemydie_hole");
                }

            }

            else
            {
                if (!TileEffect_t.has_effect(cumulative_effects, TileEffect_t.BLOCK))
                {
                    //Do the move.
                    Console.print("Would Move");
                    would_move = true;
                    tile.move(tile.tx+xsteps, tile.ty+ysteps);
                    //Console.print("this.getType(): " + this.getType());
                    //Console.print("this: " + this);
                    //if (this.getType() is Player)
                    if (tile is dungeonsofdestiny.game.Player)
                    {
                        Console.print("Playing!");
                        current_room.play_sfx("step");
                    }
                    else
                        Console.print("Wrong type");
                }
                else
                {
                    tile.blocked_last_turn = true;
                }

            }
        
            return would_move;
        }



        public function wipe_current_room()
        {
            current_room.teardown_room();
            removeChild(current_room);
            current_room = null;
            //exit();
        }

        public function create_current_room(stage_ : Stage)
        {
            current_room = new Room(stage_, _roomdefs[_current_room_number-1], sfx);
            addChild(current_room);
        }

        public function create_next_room(stage_ : Stage)
        {
            if (_current_room_number <= _roomdefs.length)
            {
                _current_room_number+=1;
                create_current_room(stage_);
            }
            else
            {
                Console.print("Wow you won!");
            }
        }
        
        public function enter(owner:DisplayObjectContainer)
        {
            super.enter(owner);
            _state = STATE_GAME;
            
            Mobile.allowScreenSleep(false);
            
            // Reset state
            _t = 0;
            soundtrack.play();
            updateMuteMode();

            stage.addEventListener(TouchEvent.TOUCH, on_touch);
            stage.addEventListener(KeyboardEvent.KEY_DOWN, on_key);
            //If there's no current room, we must have died and
            //reentered
            if (!current_room)
            {
                create_current_room(stage);
            }

            _movement_sensor.on_sense_direction += process_sense_direction;
            

        }

        

        public function exit()
        {
            //stage.removeEventListener(TouchEvent.TOUCH, on_touch);
            super.exit();
            if (_state == STATE_DEMO) {
                //config = origConfig;
                //board.touchable = true;
                //Tile.swapTime = Tile.SWAP_TIME_NORMAL;
                //demoHand.visible = false;
                //demoInstructions.visible = false;
            }
            _state = STATE_SLEEP;
            //resetJuggler();
            //particles.clear();
            soundtrack.stop();
            Mobile.allowScreenSleep(true);

        }

        public function on_touch(te: TouchEvent): void
        {
            // Filter to only consider new touches.
            var point:Point;
            
            var touch = te.getTouch(stage, TouchPhase.BEGAN);
            if (touch)
            {
                point = touch.getLocation(stage);
                //trace("Touch (,",touch.id,") x:", point.x, " y:", point.y);
                _movement_sensor.addSample(point);
            }

            touch = te.getTouch(stage, TouchPhase.MOVED);
            if (touch)
            {
                point = touch.getLocation(stage);
                //trace("MOVED (",touch.id,") x:", point.x, " y:", point.y);
                _movement_sensor.addSample(point);
            }

            touch = te.getTouch(stage, TouchPhase.ENDED);
            if (touch)
            {
                point = touch.getLocation(stage);
                //trace("ENDED (",touch.id,") x:", point.x, " y:", point.y);
                _movement_sensor.reset();
            }

        }

        public function on_key(ke: KeyboardEvent): void
        {
           //if (current_room)
           // {
                //Console.print(ke.keyCode);
                var dir = new Point();
                switch (ke.keyCode)
                {
                    case 82: //up
                        dir.x = 0;
                        dir.y = -1;
                        //process_sense_direction(dir);
                        break;
                    case 79: //right
                        dir.x = 1;
                        dir.y = 0;
                        //process_sense_direction(dir);
                        break;
                    case 81: //down
                        dir.x = 0;
                        dir.y = 1;
                        //process_sense_direction(dir);
                        break;
                    case 80: //left
                        dir.x = -1;
                        dir.y = 0;
                        //process_sense_direction(dir);
                        break;
                }
                //if (p.x != 0 && p.y != 0)
                //{
                    Console.print("kb input");
                    process_sense_direction(dir);
                //}
            //}
 
        }


        public function process_sense_direction(dir: Point) : void
        {
            trace("The direction was ", dir);
            //Let everybody know what the player's last move was
            current_room.player_move_direction = dir;
            //Only process enemy moves if it was a valid move
            if ( submit_move(current_room.player, dir.x, dir.y) )
            {
                for each (var enemy: Enemy in current_room.enemies)
                {
                    if (enemy) //just in case it is null
                    {
                        enemy.take_turn(dir);
                        var d: DungeonEntityMove = enemy.get_queued_move();
                        if (d)
                        {
                            submit_move(d.tile, d.dir_x, d.dir_y);
                            enemy.clear_queued_move();
                        }
                    }
                }
            }
        }



        public function tick()
        {
            // Do not process ticks after quitting
            if (_state == STATE_QUIT) return;
            if (_paused) return;
            
            _t += _dt;
            _juggler.advanceTime(_dt);
            
            /*
            screenshaker.strength = screenshake;
            
            // Decays
            screenshake -= screenshake*6*dt;
            if (Math.abs(screenshake) < 0.1) screenshake = 0;
            momentum -= momentum*0.2*dt;
            
            // Scroll based on momentum
            bgScroll -= momentum*1.5*dt;
            */
            // Do not process multiplier or time while in end animation
            if (_state == STATE_ENDING) return;
            
            /*
            multiplier = Math.round(Math.pow(1+0.1*momentum, 2)/0.5)*0.5;
            updateMulti();
            updateTime();
            if (config.duration > 0 && t >= config.duration) {
                end();
            }
            */
        }

        public function render()
        {

        }



        /**
         * Exists for symmetry against activate()
         */
        public function deactivate() {
            _paused = true;
        }
        
        /**
         * Update mute mode on activation,
         * keeps sound from getting unmuted unintentionally
         */
        public function activate() {
            updateMuteMode();
            _paused = false;
        }
        
        /**
         * Switch over to the next mute mode
         */
        private function switchMuteMode(e:Event) 
        {
            switch (_muteMode) {
                case MUTE_NONE:  _muteMode = MUTE_MUSIC; break;
                case MUTE_MUSIC: _muteMode = MUTE_ALL; break;
                case MUTE_ALL:   _muteMode = MUTE_NONE; break;
            }
            // Save mute mode to persistent storage
            UserDefault.sharedUserDefault().setIntegerForKey("_muteMode", _muteMode);
            updateMuteMode();
        }
        
        /**
         * Update the sounds and state based on the current mute mode
         */
        private function updateMuteMode() 
        {
            if (_muteMode == MUTE_MUSIC || _muteMode == MUTE_ALL) {
                soundtrack.pause();
            } else if (_state != STATE_SLEEP) {
                soundtrack.play();
            }
            if (_muteMode == MUTE_ALL) {
                Listener.setGain(0);
            } else {
                Listener.setGain(1);
            }
            switch (_muteMode) {
                case MUTE_NONE: mute.label = "MUTE"; break;
                case MUTE_MUSIC: mute.label = "SFX"; break;
                case MUTE_ALL: mute.label = "MUTED"; break;
            }
        }


    }
}
